import _extends from "@babel/runtime/helpers/extends";
import _defineProperty from "@babel/runtime/helpers/defineProperty";
import _objectSpread from "@babel/runtime/helpers/objectSpread2";
import _slicedToArray from "@babel/runtime/helpers/slicedToArray";
import React, { useCallback, useContext, useEffect, useMemo } from 'react';
import { action, runInAction } from 'mobx';
import { observer } from 'mobx-react-lite';
import classNames from 'classnames';
import omit from 'lodash/omit';
import isFunction from 'lodash/isFunction';
import { IteratorHelper } from 'choerodon-ui/dataset';
import { pxToRem } from '../../../es/_util/UnitConvertor';
import measureScrollbar from '../../../es/_util/measureScrollbar';
import { defaultAggregationRenderer } from './Column';
import TableContext from './TableContext';
import { getColumnLock, getEditorByColumnAndRecord, isDisabledRow, isStickySupport } from './utils';
import { ColumnLock } from './enum';
import TableCellInner from './TableCellInner';
import { treeSome } from '../_util/treeUtils';
import TableGroupCellInner from './TableGroupCellInner';
import { DRAG_KEY, SELECTION_KEY, ROW_NUMBER_KEY } from './TableStore';
import { groupedAggregationTree } from './AggregationTree';
import AggregationTreeGroups from './AggregationTreeGroups';

function getRowSpan(group, tableStore) {
  if (tableStore.headerTableGroups.length) {
    var subGroups = group.subGroups,
        subHGroups = group.subHGroups;

    if (subHGroups) {
      return IteratorHelper.iteratorReduce(subHGroups.values(), function (rowSpan, group) {
        return Math.max(rowSpan, group.records.length);
      }, 0);
    }

    return subGroups.reduce(function (rowSpan, subGroup) {
      return rowSpan + (subGroup.subGroups.length ? getRowSpan(subGroup, tableStore) : 0) + subGroup.records.length;
    }, 0);
  }

  return group.expandedRecords.length;
}

function getTdElementByTarget(target) {
  if (target.tagName.toLowerCase() !== "td") {
    return getTdElementByTarget(target.parentElement);
  }

  return target;
}

var AUTO_SCROLL_SENSITIVITY = 50; // 控制开始滚动的边距

var AUTO_SCROLL_SPEED = 20; // 滚动速率

var TableCell = function TableCell(props) {
  var _classNames, _classNames3;

  var columnGroup = props.columnGroup,
      record = props.record,
      isDragging = props.isDragging,
      provided = props.provided,
      isDragDisabled = props.isDragDisabled,
      colSpan = props.colSpan,
      className = props.className,
      children = props.children,
      disabled = props.disabled,
      groupPath = props.groupPath,
      rowIndex = props.rowIndex,
      virtualHeight = props.virtualHeight,
      intersectionRef = props.intersectionRef,
      isFixedRowHeight = props.isFixedRowHeight,
      isRenderCell = props.isRenderCell;
  var mousePosition = React.useRef(null);
  var dragDisabled = isFunction(isDragDisabled) ? isDragDisabled(record) : isDragDisabled;
  var column = columnGroup.column,
      key = columnGroup.key;

  var _useContext = useContext(TableContext),
      tableStore = _useContext.tableStore,
      prefixCls = _useContext.prefixCls,
      dataSet = _useContext.dataSet,
      expandIconAsCell = _useContext.expandIconAsCell,
      tableAggregation = _useContext.aggregation,
      rowHeight = _useContext.rowHeight;

  var clipboard = tableStore.clipboard,
      startChooseCell = tableStore.startChooseCell,
      endChooseCell = tableStore.endChooseCell,
      isFinishChooseCell = tableStore.isFinishChooseCell,
      currentEditorName = tableStore.currentEditorName,
      drawCopyBorder = tableStore.drawCopyBorder,
      dragColumnAlign = tableStore.dragColumnAlign,
      rowDraggable = tableStore.rowDraggable;
  var cellPrefix = "".concat(prefixCls, "-cell");
  var tableColumnOnCell = tableStore.getConfig('tableColumnOnCell');
  var __tableGroup = column.__tableGroup,
      style = column.style,
      lock = column.lock,
      onCell = column.onCell,
      aggregation = column.aggregation;

  var _ref = __tableGroup && groupPath && groupPath.find(function (_ref3) {
    var _ref4 = _slicedToArray(_ref3, 1),
        path = _ref4[0];

    return path.name === __tableGroup.name;
  }) || [undefined, false],
      _ref2 = _slicedToArray(_ref, 2),
      group = _ref2[0],
      isLast = _ref2[1];

  var _ref5 = group || !groupPath || !groupPath.length ? [undefined, false] : groupPath[groupPath.length - 1],
      _ref6 = _slicedToArray(_ref5, 1),
      rowGroup = _ref6[0];

  useEffect(function () {
    return function () {
      stopAutoScroll();
      document.removeEventListener('mouseup', handleDocumentMouseUp);
    };
  }, []);
  var getInnerNode = useCallback(function (col, onCellStyle, inAggregation, headerGroup) {
    return record ? /*#__PURE__*/React.createElement(TableCellInner, {
      column: col,
      record: record,
      style: onCellStyle,
      disabled: disabled,
      inAggregation: inAggregation,
      prefixCls: cellPrefix,
      colSpan: colSpan,
      headerGroup: headerGroup,
      rowGroup: rowGroup
    }, children) : undefined;
  }, [record, disabled, children, cellPrefix, colSpan, rowGroup]);
  var isBuiltInColumn = tableStore.isBuiltInColumn(column);
  var columnOnCell = !isBuiltInColumn && (onCell || tableColumnOnCell);
  var cellExternalProps = typeof columnOnCell === 'function' && record ? columnOnCell({
    dataSet: dataSet,
    record: record,
    column: column
  }) : {};
  var handleClickCapture = useCallback(action(function (e) {
    if (tableStore.currentEditorName && key === DRAG_KEY) {
      tableStore.blurEditor();
    }

    if (record && !isDisabledRow(record) && e.target.dataset.selectionKey !== SELECTION_KEY) {
      dataSet.current = record;
    }

    var onClickCapture = cellExternalProps.onClickCapture;

    if (typeof onClickCapture === 'function') {
      onClickCapture(e);
    }
  }), [record, dataSet, cellExternalProps]);
  var handleMouseDown = useCallback(action(function (event) {
    var target = event.target;
    if (key === DRAG_KEY || key === ROW_NUMBER_KEY) return; // 往上层一直找到  td  元素

    var startTarget = getTdElementByTarget(target);
    var colIndex = tableStore.columnGroups.leafs.findIndex(function (x) {
      return x.column.name === key || x.column.key === key;
    });
    if (colIndex < 0) return;

    if (startTarget) {
      var initPosition = {
        rowIndex: rowIndex,
        colIndex: colIndex,
        target: startTarget
      };

      if (tableStore.startChooseCell && event.shiftKey) {
        tableStore.shiftKey = true;
        tableStore.endChooseCell = initPosition;
      } else {
        tableStore.shiftKey = false;
        tableStore.startChooseCell = initPosition;
        tableStore.endChooseCell = initPosition;
      }

      drawCopyBorder(tableStore.startChooseCell.target, startTarget);
      tableStore.isFinishChooseCell = false;
      document.addEventListener('mouseup', handleDocumentMouseUp, {
        once: true
      });
    }
  }), [endChooseCell]);
  var handleDocumentMouseUp = useCallback(action(function () {
    tableStore.isFinishChooseCell = true;
    stopAutoScroll();
  }), []);
  var handleMouseOver = useCallback(action(function (event) {
    if (rowDraggable && !dragColumnAlign || key === DRAG_KEY || key === ROW_NUMBER_KEY) return;

    if (startChooseCell && !isFinishChooseCell && !currentEditorName) {
      var colIndex = tableStore.columnGroups.leafs.findIndex(function (x) {
        return x.column.name === key || x.column.key === key;
      });

      if (colIndex >= 0) {
        var startTarget = startChooseCell.target;
        var endTarget = getTdElementByTarget(event.target);
        tableStore.endChooseCell = {
          colIndex: colIndex,
          rowIndex: rowIndex,
          target: endTarget
        };

        if (endTarget) {
          drawCopyBorder(startTarget, endTarget);
        }
      }

      autoScroll(event);
    }
  }), [startChooseCell, isFinishChooseCell, currentEditorName]);

  var autoScroll = function autoScroll(event) {
    // 控制滚动条自动滚动
    var node = tableStore.node,
        rightColumnGroups = tableStore.rightColumnGroups,
        leftColumnGroups = tableStore.leftColumnGroups,
        overflowY = tableStore.overflowY,
        overflowX = tableStore.overflowX,
        lastScrollLeft = tableStore.lastScrollLeft,
        lastScrollTop = tableStore.lastScrollTop;
    var tableContentWrap = node.tableContentWrap,
        tableBodyWrap = node.tableBodyWrap;
    var overflowWrapper = tableBodyWrap || tableContentWrap;
    if (!overflowWrapper) return;
    var tableRect = overflowWrapper.getBoundingClientRect();
    var overflowYWidth = overflowY ? measureScrollbar() : 0;
    var height = tableRect.height,
        width = tableRect.width,
        x = tableRect.x,
        y = tableRect.y;
    var mouseX = event.clientX - x;
    var mouseY = event.clientY - y;
    mousePosition.current = {
      x: mouseX,
      y: mouseY
    };
    var maxScrollLeft = overflowWrapper.scrollWidth - overflowWrapper.clientWidth;
    var maxScrollHeight = overflowWrapper.scrollHeight - overflowWrapper.clientHeight;
    var hasEnteredVerticalSensitivityArea = mouseY <= AUTO_SCROLL_SENSITIVITY && lastScrollTop !== 0 || lastScrollTop !== maxScrollHeight && mouseY >= height - AUTO_SCROLL_SENSITIVITY - (overflowX ? measureScrollbar('horizontal') : 0);
    var hasEnteredHorizontalSensitivityArea = mouseX >= leftColumnGroups.width - AUTO_SCROLL_SENSITIVITY && mouseX <= AUTO_SCROLL_SENSITIVITY + leftColumnGroups.width || maxScrollLeft !== lastScrollLeft && mouseX >= width - rightColumnGroups.width - AUTO_SCROLL_SENSITIVITY - overflowYWidth && mouseX <= width - rightColumnGroups.width + AUTO_SCROLL_SENSITIVITY - overflowYWidth;
    var hasEnteredSensitivityArea = hasEnteredVerticalSensitivityArea || hasEnteredHorizontalSensitivityArea;

    if (hasEnteredSensitivityArea) {
      // 鼠标在sensitivity区域，开始自动滚动
      startAutoScroll(overflowWrapper);
    } else {
      // 鼠标离开sensitivity区域，停止自动滚动
      stopAutoScroll();
    }
  };

  var startAutoScroll = function startAutoScroll(overflowWrapper) {
    if (tableStore.autoScrollRAF || !mousePosition.current) {
      return;
    }

    var execScroll = action(function () {
      var rightColumnGroups = tableStore.rightColumnGroups,
          leftColumnGroups = tableStore.leftColumnGroups,
          overflowX = tableStore.overflowX,
          overflowY = tableStore.overflowY,
          lastScrollLeft = tableStore.lastScrollLeft,
          lastScrollTop = tableStore.lastScrollTop;
      var _mousePosition$curren = mousePosition.current,
          mouseX = _mousePosition$curren.x,
          mouseY = _mousePosition$curren.y;

      var _overflowWrapper$getB = overflowWrapper.getBoundingClientRect(),
          height = _overflowWrapper$getB.height,
          width = _overflowWrapper$getB.width;

      var deltaX = 0;
      var deltaY = 0;
      var factor = 0;
      var scrollVerticalWidth = overflowY ? measureScrollbar() : 0;
      var scrollHorizontalWidth = overflowX ? measureScrollbar('horizontal') : 0;
      var maxScrollLeft = overflowWrapper.scrollWidth - overflowWrapper.clientWidth;
      var maxScrollHeight = overflowWrapper.scrollHeight - overflowWrapper.clientHeight;

      if (mouseY <= AUTO_SCROLL_SENSITIVITY && overflowY && lastScrollTop !== 0) {
        // 向上滚动，距离越近，滚动的速度越快
        factor = (AUTO_SCROLL_SENSITIVITY - mouseY) / -AUTO_SCROLL_SENSITIVITY;
        deltaY = AUTO_SCROLL_SPEED;
      } else if (mouseY >= height - scrollHorizontalWidth - AUTO_SCROLL_SENSITIVITY && overflowY && maxScrollHeight !== lastScrollTop) {
        // 向下滚动，距离越近，滚动的速度越快
        factor = (mouseY - (height - scrollHorizontalWidth - AUTO_SCROLL_SENSITIVITY)) / AUTO_SCROLL_SENSITIVITY;
        deltaY = AUTO_SCROLL_SPEED;
      } else if (mouseX - leftColumnGroups.width <= AUTO_SCROLL_SENSITIVITY && overflowX) {
        // 滚动到最左边，距离越近，滚动的速度越快
        factor = (AUTO_SCROLL_SENSITIVITY - mouseX + leftColumnGroups.width) / -AUTO_SCROLL_SENSITIVITY;
        deltaX = AUTO_SCROLL_SPEED;
      } else if (mouseX >= width - rightColumnGroups.width - scrollVerticalWidth - AUTO_SCROLL_SENSITIVITY && overflowX && maxScrollLeft !== lastScrollLeft) {
        // 滚动到最右边，距离越近，滚动的速度越快
        factor = (mouseX - (width - rightColumnGroups.width - AUTO_SCROLL_SENSITIVITY - scrollVerticalWidth)) / AUTO_SCROLL_SENSITIVITY;
        deltaX = AUTO_SCROLL_SPEED;
      }

      if (deltaX !== 0 || deltaY !== 0) {
        overflowWrapper.scrollTo({
          left: lastScrollLeft + deltaX * factor,
          top: lastScrollTop + deltaY * factor
        });
      }

      tableStore.autoScrollRAF = requestAnimationFrame(execScroll);
    });
    execScroll();
  };

  var stopAutoScroll = function stopAutoScroll() {
    if (tableStore.autoScrollRAF) {
      cancelAnimationFrame(tableStore.autoScrollRAF);
      runInAction(function () {
        tableStore.autoScrollRAF = null;
      });
    }
  };

  var handleMouseUp = useCallback(action(function () {
    tableStore.isFinishChooseCell = true;
    stopAutoScroll();
  }), []);
  var isChoose = useMemo(function () {
    var colIndex = tableStore.columnGroups.leafs.findIndex(function (x) {
      return x.column.name === key || x.column.key === key;
    });

    if (startChooseCell && startChooseCell.colIndex === colIndex && startChooseCell.rowIndex === rowIndex) {
      return true;
    }

    if (endChooseCell && endChooseCell.colIndex === colIndex && endChooseCell.rowIndex === rowIndex) {
      return true;
    }

    if (endChooseCell && startChooseCell) {
      var finalStartRowIndex = startChooseCell.rowIndex;
      var finalStartColumnIndex = startChooseCell.colIndex;
      var finalEndRowIndex = endChooseCell.rowIndex;
      var finalEndColumnIndex = endChooseCell.colIndex;

      if (finalStartRowIndex > finalEndRowIndex) {
        finalStartRowIndex = endChooseCell.rowIndex;
        finalEndRowIndex = startChooseCell.rowIndex;
      }

      if (finalStartColumnIndex > finalEndColumnIndex) {
        finalStartColumnIndex = endChooseCell.colIndex;
        finalEndColumnIndex = startChooseCell.colIndex;
      }

      if (colIndex >= finalStartColumnIndex && colIndex <= finalEndColumnIndex && rowIndex >= finalStartRowIndex && rowIndex <= finalEndRowIndex) {
        return true;
      }
    }

    return false;
  }, [startChooseCell, endChooseCell]);
  var isFirstChoosed = useMemo(function () {
    var colIndex = tableStore.columnGroups.leafs.findIndex(function (x) {
      return x.column.name === key || x.column.key === key;
    });
    return startChooseCell && startChooseCell.rowIndex === rowIndex && startChooseCell.colIndex === colIndex;
  }, [startChooseCell]);
  var aggregationTreeRenderer = useCallback(function (_ref7) {
    var colGroup = _ref7.colGroup,
        style = _ref7.style;
    return getInnerNode(colGroup.column, style, true, colGroup.headerGroup);
  }, [getInnerNode]);
  var columnLock = isStickySupport() && getColumnLock(lock);

  var baseStyle = function () {
    if (columnLock) {
      if (columnLock === ColumnLock.left) {
        return _objectSpread(_objectSpread({}, style), {}, {
          left: pxToRem(columnGroup.left, true)
        });
      }

      if (columnLock === ColumnLock.right) {
        return _objectSpread(_objectSpread({}, style), {}, {
          right: pxToRem(colSpan && colSpan > 1 ? 0 : columnGroup.right, true)
        });
      }
    }

    return style;
  }();

  var baseClassName = classNames(cellPrefix, (_classNames = {}, _defineProperty(_classNames, "".concat(cellPrefix, "-fix-").concat(columnLock), columnLock), _defineProperty(_classNames, "".concat(cellPrefix, "-no-transition"), !tableStore.tableColumnResizeTransition), _classNames));
  var intersectionProps = {};

  if (intersectionRef) {
    intersectionProps.ref = intersectionRef;

    if (virtualHeight !== undefined) {
      intersectionProps.style = _objectSpread(_objectSpread({}, baseStyle), {}, {
        height: virtualHeight
      });
    }
  }

  if (!record) {
    return /*#__PURE__*/React.createElement("td", _extends({
      className: baseClassName,
      style: baseStyle,
      "data-index": key,
      colSpan: colSpan
    }, intersectionProps));
  }

  var indexInGroup = group ? group.expandedRecords.indexOf(record) : -1;
  var groupCell = indexInGroup === 0 || tableStore.virtual && indexInGroup > -1 && tableStore.virtualStartIndex === rowIndex;

  if (group && !groupCell) {
    return null;
  }

  var getAggregationTreeGroups = function getAggregationTreeGroups($aggregation) {
    if ($aggregation) {
      var childrenInAggregation = columnGroup.childrenInAggregation;

      if (childrenInAggregation) {
        var groups = childrenInAggregation.columns;
        return groupedAggregationTree({
          rowGroup: __tableGroup ? group : rowGroup,
          headerGroup: columnGroup.headerGroup,
          record: record,
          groups: groups,
          column: column,
          renderer: aggregationTreeRenderer
        });
      }
    }
  };

  var groupRowSpan = groupCell && group ? getRowSpan(group, tableStore) - indexInGroup : undefined;
  var rowSpan = groupRowSpan === 1 ? undefined : groupRowSpan;

  var renderInnerNode = function renderInnerNode($aggregation, onCellStyle) {
    if (expandIconAsCell && children) {
      return /*#__PURE__*/React.createElement("span", {
        className: classNames("".concat(cellPrefix, "-inner"), _defineProperty({}, "".concat(cellPrefix, "-inner-row-height-fixed"), rowHeight !== 'auto')),
        style: _objectSpread({
          textAlign: 'center'
        }, onCellStyle)
      }, children);
    }

    var aggregationList = getAggregationTreeGroups($aggregation);

    if (groupCell && group && __tableGroup) {
      return /*#__PURE__*/React.createElement(TableGroupCellInner, {
        rowSpan: rowSpan,
        group: group,
        column: column,
        isFixedRowHeight: isFixedRowHeight
      }, aggregationList);
    }

    if (aggregationList) {
      var _column$renderer = column.renderer,
          renderer = _column$renderer === void 0 ? defaultAggregationRenderer : _column$renderer;
      var treeGroups = /*#__PURE__*/React.createElement(AggregationTreeGroups, {
        trees: aggregationList
      });
      return /*#__PURE__*/React.createElement("span", {
        className: "".concat(cellPrefix, "-inner")
      }, renderer({
        text: treeGroups,
        record: record,
        dataSet: dataSet,
        aggregation: tableAggregation,
        headerGroup: columnGroup.headerGroup,
        rowGroup: rowGroup,
        aggregationTree: aggregationList
      }));
    }

    return getInnerNode(column, onCellStyle, undefined, columnGroup.headerGroup);
  };

  var scope = groupCell ? 'row' : undefined;
  var TCell = scope ? 'th' : 'td';

  if (!isRenderCell) {
    var hasEditor = aggregation ? treeSome(column.children || [], function (node) {
      return !!getEditorByColumnAndRecord(node, record);
    }) : !!getEditorByColumnAndRecord(column, record);

    var emptyCellProps = _objectSpread(_objectSpread({
      colSpan: colSpan,
      rowSpan: rowSpan,
      'data-index': key
    }, cellExternalProps), {}, {
      className: classNames(baseClassName, cellExternalProps.className),
      scope: scope,
      onClickCapture: handleClickCapture
    });

    if (hasEditor) {
      emptyCellProps.onFocus = function (e) {
        tableStore.activeEmptyCell = e.target;
      };

      emptyCellProps.onBlur = function () {
        delete tableStore.activeEmptyCell;
      };

      emptyCellProps.tabIndex = 0;
    }

    return /*#__PURE__*/React.createElement(TCell, _extends({}, emptyCellProps, intersectionProps, {
      style: _objectSpread(_objectSpread(_objectSpread({}, baseStyle), cellExternalProps.style), intersectionProps.style)
    }));
  }

  var cellStyle = _objectSpread(_objectSpread(_objectSpread({}, baseStyle), cellExternalProps.style), provided && {
    cursor: dragDisabled ? 'not-allowed' : 'move'
  });

  var classString = classNames(baseClassName, (_classNames3 = {}, _defineProperty(_classNames3, "".concat(cellPrefix, "-aggregation"), aggregation), _defineProperty(_classNames3, "".concat(cellPrefix, "-last-group"), groupCell && isLast), _defineProperty(_classNames3, "".concat(cellPrefix, "-choosed"), clipboard && clipboard.copy && isChoose), _defineProperty(_classNames3, "".concat(cellPrefix, "-first-choosed"), clipboard && clipboard.copy && isFirstChoosed), _classNames3), column.className, className, cellExternalProps.className);

  var widthDraggingStyle = function widthDraggingStyle() {
    var draggingStyle = {};

    if (isDragging) {
      var dom = tableStore.node.element.querySelector(".".concat(prefixCls, "-tbody .").concat(prefixCls, "-cell[data-index=\"").concat(key, "\"]"));

      if (dom) {
        draggingStyle.width = dom.clientWidth;
        draggingStyle.whiteSpace = 'nowrap';
      }
    }

    return draggingStyle;
  }; // 只有全局属性时候的样式可以继承给下级满足对td的样式能够一致表现


  var onCellStyle = !isBuiltInColumn && tableColumnOnCell === columnOnCell && typeof tableColumnOnCell === 'function' ? omit(cellExternalProps.style, ['width', 'height']) : undefined;
  var clipboardCopyEvents = {};

  if (clipboard && clipboard.copy) {
    clipboardCopyEvents = {
      onMouseDown: handleMouseDown,
      onMouseOver: handleMouseOver,
      onMouseUp: handleMouseUp
    };
  }

  return /*#__PURE__*/React.createElement(TCell, _extends({
    colSpan: colSpan,
    rowSpan: rowSpan
  }, cellExternalProps, {
    className: classString,
    "data-index": key
  }, provided && provided.dragHandleProps, intersectionProps, {
    style: _objectSpread(_objectSpread(_objectSpread({}, omit(cellStyle, ['width', 'height'])), widthDraggingStyle()), intersectionProps.style),
    scope: scope,
    onClickCapture: handleClickCapture
  }, clipboardCopyEvents), renderInnerNode(aggregation, onCellStyle));
};

TableCell.displayName = 'TableCell';
export default observer(TableCell);
//# sourceMappingURL=TableCell.js.map
